# © Copyright Databand.ai, an IBM Company 2022


def run_task(task):
    run = task.dbnd_run()
    from IPython.core.display import HTML

    return HTML(
        'You can review Databand Run <a href="%s" target="_blank" >at this link</a>'
        % run.run_url
    )
